<?php

namespace SGalinski\TypoScriptReferenceFrontend\Service;

use SGalinski\TypoScriptReferenceFrontend\Domain\Model\Attribute;
use TYPO3\Flow\Annotations as Flow;

/**
 * The Class TsrefRestService contains methods for communication with TypoScript REST API.
 *
 * @package SGalinski\TypoScriptReferenceFrontend\Service
 * @Flow\Scope("singleton")
 */
class TsrefRestService {
	/**
	 * URL of REST API resource.
	 */
	const RESOURCE_URL = '127.0.0.1:8000/api/attributes';

	/**
	 * Default typo3 version of typoScript attributes
	 */
	const TYPO3_DEFAULT_VERSION = '6.2';

	/**
	 * Makes curl handle, sets some general options and returns the handle.
	 * It also checks if cURL is installed.
	 *
	 * @param string $suffix
	 * @return resource
	 */
	protected function initialiseCurl($suffix = '') {
		// is cURL installed yet?
		if (!function_exists('curl_init')) {
			throw new \RuntimeException('Sorry cURL is not installed. Install cURL.');
		}

		$curlHandle = curl_init();

		// Set URL to download
		curl_setopt($curlHandle, CURLOPT_URL, self::RESOURCE_URL . $suffix);

		// Should cURL return or print out the data? (true = return, false = print)
		curl_setopt($curlHandle, CURLOPT_RETURNTRANSFER, TRUE);

		// Timeout in seconds
		curl_setopt($curlHandle, CURLOPT_TIMEOUT, 10);
		return $curlHandle;
	}

	/**
	 * @param boolean|null $isType
	 * @param string $typo3Version
	 * @param int $typo3Group
	 * @return mixed
	 */
	public function getAttributesJson(
		$isType = NULL, $typo3Version = self::TYPO3_DEFAULT_VERSION, $typo3Group = Attribute::NORMAL_GROUP
	) {
		$curlHandle = $this->initialiseCurl();

		$headerProperties = [
			'version: ' . $typo3Version,
			'typo3group: ' . $typo3Group
		];

		if ($isType !== NULL) {
			$headerProperties[] = 'istype: ' . ($isType ? '1' : '0');
		}

		curl_setopt($curlHandle, CURLOPT_HTTPHEADER, $headerProperties);

		// Download the given URL, and return output
		$output = curl_exec($curlHandle);

		// Close the cURL resource, and free system resources
		curl_close($curlHandle);

		return $output;
	}

	/**
	 * Returns an attribute by its ID from RESOURCE_URL.
	 *
	 * @param int $attributeId
	 * @return mixed
	 */
	public function getAttributeByIdJson($attributeId) {
		$curlHandle = $this->initialiseCurl('/' . $attributeId);

		// Download the given URL, and return output
		$output = curl_exec($curlHandle);

		// Close the cURL resource, and free system resources
		curl_close($curlHandle);

		return $output;
	}

	/**
	 * Returns properties of a type (attribute which is a type) by its ID from RESOURCE_URL/ID/properties.
	 *
	 * @param int $attributeId
	 * @return mixed
	 */
	public function getPropertiesByParentIdJson($attributeId) {
		$curlHandle = $this->initialiseCurl('/' . $attributeId . '/properties');

		// Download the given URL, and return output
		$output = curl_exec($curlHandle);

		// Close the cURL resource, and free system resources
		curl_close($curlHandle);

		return $output;
	}

	/**
	 * Gets all type attributes by $typo3Version and $typo3Group from RESOURCE_URL
	 *
	 * @param int $attributeId
	 * @return mixed
	 */
	public function getAttributeById($attributeId) {
		$attributeJson = $this->getPropertiesByParentIdJson($attributeId);
		$attributePhp = json_decode($attributeJson);
		return $attributePhp;
	}

	/**
	 * Gets all type attributes by $typo3Version and $typo3Group from RESOURCE_URL
	 *
	 * @param int $attributeId
	 * @return mixed
	 */
	public function getPropertiesByParentId($attributeId) {
		$propertiesJson = $this->getPropertiesByParentIdJson($attributeId);
		$propertiesPhp = json_decode($propertiesJson);
		return $propertiesPhp;
	}

	/**
	 * Gets all type attributes by $typo3Version and $typo3Group from RESOURCE_URL
	 *
	 * @param string $typo3Version
	 * @param int $typo3Group
	 * @return mixed
	 */
	public function getTypes($typo3Version = self::TYPO3_DEFAULT_VERSION, $typo3Group = Attribute::NORMAL_GROUP) {
		$typesJson = $this->getAttributesJson(TRUE, $typo3Version, $typo3Group);
		$typesPhp = json_decode($typesJson);
		return $typesPhp;
	}

	/**
	 * Adds new attribute to REST API resource by use of POST method.
	 *
	 * @param Attribute $attribute
	 */
	public function addNewAttribute(Attribute $attribute) {

	}
}